/**
   \author Shin'ichiro Nakaoka
*/

#include "OSGSceneGL.h"
#include <osg/Geode>

using namespace osg;
using namespace cnoid;

namespace cnoid {

    class OSGSceneGLDrawable : public osg::Drawable
    {
    public:
        OSGSceneGLDrawable()
        {
            owner = 0;
        }

        OSGSceneGLDrawable(const OSGSceneGLDrawable& org, const osg::CopyOp& copyop = osg::CopyOp::SHALLOW_COPY)
            : osg::Drawable(org, copyop)
        {
            owner = 0;
        }

        META_Object(cnoidBase, OSGSceneGLDrawable)

        virtual ~OSGSceneGLDrawable() { }

        virtual void drawImplementation(osg::RenderInfo& ri) const {
            owner->drawImplementation(ri);
        }
        
        virtual osg::BoundingBox computeBound() const {
            return owner->computeGLBound();
        }

        virtual void accept(osg::PrimitiveFunctor& functor) const  {
            owner->accept(functor);
        }

        OSGSceneGL* owner;
    };
}


OSGSceneGL::OSGSceneGL()
{
    initializeDrawable(new OSGSceneGLDrawable());
}


OSGSceneGL::OSGSceneGL(const OSGSceneGL& org, const osg::CopyOp& copyop)
    : OSGSceneObject(org, copyop)
{
    initializeDrawable(new OSGSceneGLDrawable(*org.drawable, copyop));
}


void OSGSceneGL::initializeDrawable(OSGSceneGLDrawable* drawable)
{
    this->drawable = drawable;
    drawable->owner = this;
    osg::Geode* geode = new osg::Geode();
    geode->addDrawable(drawable);
    addChild(geode);
}


OSGSceneGL::~OSGSceneGL()
{

}


void OSGSceneGL::setUseDisplayList(bool on)
{
    drawable->setUseDisplayList(on);
}


void OSGSceneGL::dirtyDisplayList()
{
    drawable->dirtyDisplayList();
    drawable->dirtyBound();
    requestRedraw();
}


osg::BoundingBox OSGSceneGL::computeGLBound() const
{
    return osg::BoundingBox();
}


/**
   Reimplement this function and specify vertices using the functor object
   when you need the intersection test on the objects
   rendered by custom drawImplementation().
*/
void OSGSceneGL::accept(osg::PrimitiveFunctor& functor) const
{
    drawable->osg::Drawable::accept(functor);
}
