/**
   @author Shin'ichiro Nakaoka
*/

#include "OSGSceneObject.h"
#include "OSGSceneView.h"
#include "MessageView.h"
#include <osgDB/ReadFile>

using namespace cnoid;


OSGSceneViewEvent::OSGSceneViewEvent()
    : point_(0.0, 0.0, 0.0)
{
    x_ = 0.0;
    y_ = 0.0;
    px_ = 0.0;
    py_ = 0.0;
    key_ = 0;
    modKeyMask_ = 0;
    button_ = 0;
    scrollingMotion_ = osgGA::GUIEventAdapter::SCROLL_NONE;
    scrollingDelta_ = 0.0;
}


OSGSceneViewEvent::OSGSceneViewEvent(const OSGSceneViewEvent& org)
    : camera_(org.camera_),
      point_(org.point_),
      path_(org.path_)
{
    x_ = org.x_;
    y_ = org.y_;
    px_ = org.px_;
    py_ = org.py_;
    key_ = org.key_;
    modKeyMask_ = org.modKeyMask_;
    button_ = org.button_;
    scrollingMotion_ = org.scrollingMotion_;
    scrollingDelta_ = org.scrollingDelta_;
}


void OSGSceneViewEvent::updateIndicator(const std::string& text) const
{
    sceneView->updateIndicator(text);
}


OSGSceneObject::OSGSceneObject()
{
    isActive_ = false;
    isEditable_ = false;
    sceneMode_ = VIEW_MODE;
}


OSGSceneObject::OSGSceneObject(const OSGSceneObject& org, const osg::CopyOp& copyop)
    : osg::Group(org),
      isEditable_(org.isEditable_),
      sceneMode_(org.sceneMode_)
{
    isActive_ = false;
}


OSGSceneObject::~OSGSceneObject()
{
    if(isActive_){
        onDetachedFromScene();
    }
}


const char* OSGSceneObject::libraryName() const
{
    return "cnoid";
}


const char* OSGSceneObject::className() const
{
    return "OSGSceneObject";
}


OSGSceneObject::ReadResult OSGSceneObject::load(const std::string filename)
{
    MessageView::mainInstance()->beginStdioRedirect();
    
    /**
       Probably error message can be obtained by using osgDB::Registry class.
       First, get a ReaderWriter object with getReaderWriterForExtension(),
       and use it to load a file.
       Messages can be obtained from ReadResult object.
       Let's see the implementation of osgDB::readNodeFile(), too.
    */
    osg::Node* node = osgDB::readNodeFile(filename);
    
    if(node){
        addChild(node);
        return osgDB::ReaderWriter::ReadResult(osgDB::ReaderWriter::ReadResult::FILE_LOADED);
    }

    return osgDB::ReaderWriter::ReadResult(osgDB::ReaderWriter::ReadResult::ERROR_IN_READING_FILE);

    MessageView::mainInstance()->endStdioRedirect();
}


void OSGSceneObject::show(bool on)
{
    OSGSceneView::showSceneObject(this, on);
}


void OSGSceneObject::hide()
{
    OSGSceneView::showSceneObject(this, false);
}


void OSGSceneObject::setEditable(bool on)
{
    isEditable_ = on;
}


bool OSGSceneObject::isEditable()
{
    return isEditable_;
}


void OSGSceneObject::requestRedraw()
{
    OSGSceneView::requestRedrawAll();
}


void OSGSceneObject::onAttachedToScene()
{
    
}


void OSGSceneObject::onDetachedFromScene()
{

}


bool OSGSceneObject::onKeyPressEvent(const OSGSceneViewEvent& event)
{
    return false;
}


bool OSGSceneObject::onKeyReleaseEvent(const OSGSceneViewEvent& event)
{
    return false;
}


bool OSGSceneObject::onButtonPressEvent(const OSGSceneViewEvent& event)
{
    return false;
}


bool OSGSceneObject::onButtonReleaseEvent(const OSGSceneViewEvent& event)
{
    return false;
}


bool OSGSceneObject::onDoubleClickEvent(const OSGSceneViewEvent& event)
{
    return false;
}


bool OSGSceneObject::onPointerMoveEvent(const OSGSceneViewEvent& event)
{
    return false;
}


void OSGSceneObject::onPointerLeaveEvent(const OSGSceneViewEvent& event)
{

}


bool OSGSceneObject::onScrollEvent(const OSGSceneViewEvent& event)
{
    return false;
}


void OSGSceneObject::onSceneModeChanged()
{

}


bool OSGSceneObject::onUndoRequest()
{
    return false;
}


bool OSGSceneObject::onRedoRequest()
{
    return false;
}

       
void OSGSceneObject::onContextMenuRequest(const OSGSceneViewEvent& event, MenuManager& menuManager)
{

}
