/**
   \file
   \author Shin'ichiro Nakaoka
*/

#include "Camera.h"

using namespace cnoid;

Camera::Camera()
{
    on_ = true;
    imageWidth_ = 640;
    imageHeight_ = 480;
    fieldOfView_ = 0.785398;
}


Camera::Camera(const Camera& org, bool stateOnly)
    : Device(org, stateOnly)
{
    copyStateFrom(org);
}

        
void Camera::copyStateFrom(const Camera& other)
{
    on_ = other.on_;
    imageWidth_ = other.imageWidth_;
    imageHeight_ = other.imageHeight_;
    fieldOfView_ = other.fieldOfView_;
}


bool Camera::copyStateFrom(const DeviceState& other)
{
    if(const Camera* camera = dynamic_cast<const Camera*>(&other)){
        copyStateFrom(*camera);
        return true;
    }
    return false;
}


DeviceState* Camera::cloneState()
{
    return new Camera(*this, true);
}


Device* Camera::clone()
{
    return new Camera(*this);
}


void Camera::forEachActualType(boost::function<bool(const std::type_info& type)> func)
{
    if(!func(typeid(Camera))){
        Device::forEachActualType(func);
    }
}
