/**
   \file
   \author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODY_LEGGED_BODY_HELPER_H_INCLUDED
#define CNOID_BODY_LEGGED_BODY_HELPER_H_INCLUDED

#include "Body.h"
#include "exportdecl.h"

namespace cnoid {

    class CNOID_EXPORT LeggedBodyHelper
    {
      public:
        LeggedBodyHelper();
        LeggedBodyHelper(BodyPtr body);
        LeggedBodyHelper(const LeggedBodyHelper& org);

        bool isValid() const { return isValid_; }
        virtual bool resetBody(BodyPtr body);
        
        virtual ~LeggedBodyHelper();

        Body* body() const { return body_.get(); }

        int numFeet() const { return footInfos.size(); }

        struct FootInfo {
            Link* link;
            Link* kneePitchJoint;
            Vector3 homeCop;
            Vector3 soleCenter;
            MappingPtr info;
        };

        Link* footLink(int index) const { return footInfos[index].link; }
        const FootInfo& footInfo(int index) const { return footInfos[index]; }

        Link* kneePitchJoint(int index) const { return footInfos[index].kneePitchJoint; }
        
        bool doLegIkToMoveCm(const Vector3& c, bool onlyProjectionToFloor = false);
        bool setStance(double width, Link* baseLink);

        Vector3 centerOfSole(int footIndex) const;
        Vector3 centerOfSoles() const;

        Vector3 homeCopOfSole(int footIndex) const;
        Vector3 homeCopOfSoles() const;
        
        static bool checkBodyInfoAsLeggedBody(const MappingPtr info);

      private:
        BodyPtr body_;
        bool isValid_;
        std::vector<FootInfo> footInfos;
    };

    typedef boost::shared_ptr<LeggedBodyHelper> LeggedBodyHelperPtr;

    CNOID_EXPORT LeggedBodyHelperPtr getLeggedBodyHelper(BodyPtr body);
}

#endif
