/**
   \file
   \author Shin'ichiro Nakaoka
*/

#include "Light.h"

using namespace cnoid;


Light::Light()
{
    on_ = true;
    color_.setConstant(1.0f);
    intensity_ = 1.0f;
}


Light::Light(const Light& org, bool stateOnly)
    : Device(org, stateOnly)
{
    copyStateFrom(org);
}


void Light::copyStateFrom(const Light& other)
{
    on_ = other.on_;
    color_ = other.color_;
    intensity_ = other.intensity_;
}


void Light::forEachActualType(boost::function<bool(const std::type_info& type)> func)
{
    if(!func(typeid(Light))){
        Device::forEachActualType(func);
    }
}


PointLight::PointLight()
{
    attenuation_ << 1.0f, 0.0f, 0.0f;
}


PointLight::PointLight(const PointLight& org, bool stateOnly)
    : Light(org, stateOnly)
{
    copyStateFrom(org);
}


DeviceState* PointLight::cloneState()
{
    return new PointLight(*this, true);
}


Device* PointLight::clone()
{
    return new PointLight(*this);
}


void PointLight::forEachActualType(boost::function<bool(const std::type_info& type)> func)
{
    if(!func(typeid(PointLight))){
        Light::forEachActualType(func);
    }
}


void PointLight::copyStateFrom(const PointLight& other)
{
    Light::copyStateFrom(other);
    attenuation_ = other.attenuation_;
}


bool PointLight::copyStateFrom(const DeviceState& other)
{
    if(const PointLight* light = dynamic_cast<const PointLight*>(&other)){
        copyStateFrom(*light);
        return true;
    }
    return false;
}


SpotLight::SpotLight()
{
    direction_ << 0.0f, 0.0f, -1.0f;
    beamWidth_ = 1.570796f;
    cutOffAngle_ = 0.785398;
}


SpotLight::SpotLight(const SpotLight& org, bool stateOnly)
    : PointLight(org, stateOnly)
{
    copyStateFrom(org);
}


DeviceState* SpotLight::cloneState()
{
    return new SpotLight(*this, true);
}


Device* SpotLight::clone()
{
    return new SpotLight(*this);
}


void SpotLight::forEachActualType(boost::function<bool(const std::type_info& type)> func)
{
    if(!func(typeid(SpotLight))){
        PointLight::forEachActualType(func);
    }
}


void SpotLight::copyStateFrom(const SpotLight& other)
{
    PointLight::copyStateFrom(other);
    direction_ = other.direction_;
    beamWidth_ = other.beamWidth_;
    cutOffAngle_ = other.cutOffAngle_;
}


bool SpotLight::copyStateFrom(const DeviceState& other)
{
    if(const SpotLight* light = dynamic_cast<const SpotLight*>(&other)){
        copyStateFrom(*light);
        return true;
    }
    return false;
}

