/**
   \file
   \author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODY_LIGHT_H_INCLUDED
#define CNOID_BODY_LIGHT_H_INCLUDED

#include "Device.h"
#include "exportdecl.h"

namespace cnoid {

    class CNOID_EXPORT Light : public Device
    {
      protected:
        Light();
        Light(const Light& org, bool stateOnly = false);
        
      public:
        virtual void forEachActualType(boost::function<bool(const std::type_info& type)> func);

        void copyStateFrom(const Light& other);
        
        bool on() const { return on_; }
        void on(bool on) { on_ = on; }

        const Vector3& color() const { return color_; }
        void setColor(const Vector3& c) { color_ = c; }

        double intensity() const { return intensity_; }
        void setIntensity(double intensity) { intensity_ = intensity; }
        
      private:
        bool on_;
        Vector3 color_;
        double intensity_;
    };

    typedef boost::intrusive_ptr<Light> LightPtr;


    class CNOID_EXPORT PointLight : public Light
    {
      public:
        PointLight();
        PointLight(const PointLight& org, bool stateOnly = false);

        virtual DeviceState* cloneState();
        virtual Device* clone();
        virtual void forEachActualType(boost::function<bool(const std::type_info& type)> func);
        virtual bool copyStateFrom(const DeviceState& other);
        void copyStateFrom(const PointLight& other);

        const Vector3& attenuation() const { return attenuation_; }
        void setAttenuation(const Vector3& a) { attenuation_ = a; }

      private:
        Vector3 attenuation_;
    };

    typedef boost::intrusive_ptr<PointLight> PointLightPtr;
    

    class CNOID_EXPORT SpotLight : public PointLight
    {
      public:
        SpotLight();
        SpotLight(const SpotLight& org, bool stateOnly = false);

        virtual DeviceState* cloneState();
        virtual Device* clone();
        virtual void forEachActualType(boost::function<bool(const std::type_info& type)> func);
        virtual bool copyStateFrom(const DeviceState& other);
        void copyStateFrom(const SpotLight& other);

        const Vector3& direction() const { return direction_; }
        void setDirection(const Vector3& direction) { direction_ = direction; }

        double beamWidth() const { return beamWidth_; }
        void setBeamWidth(double beamWidth) { beamWidth_ = beamWidth; }

        double cutOffAngle() const { return cutOffAngle_; }
        void setCutOffAngle(double angle) { cutOffAngle_ = angle; }
        
      private:
        Vector3 direction_;
        double beamWidth_;
        double cutOffAngle_;
    };

    typedef boost::intrusive_ptr<SpotLight> SpotLightPtr;
        
};


#endif
