/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODYPLUGIN_OSG_SCENE_WORLD_H_INCLUDED
#define CNOID_BODYPLUGIN_OSG_SCENE_WORLD_H_INCLUDED

#include <map>
#include <iosfwd>
#include <boost/signals.hpp>
#include <cnoid/OSGSceneObject>
#include "WorldItem.h"
#include "OSGCollision.h"
#include "exportdecl.h"

namespace cnoid {

    class OSGSceneView;
    class ItemTreeView;
    
    /**
       The role of this class is the visualization of the WorldItem state.
       The bodies contained in a world are visualized by the OSGSceneBody class,
       so that this class visualizes the remaining objects such as the collisions
       between bodies.
    */
    class CNOID_EXPORT OSGSceneWorld : public OSGSceneObject
    {
    public:
        OSGSceneWorld(WorldItemPtr worldItem);

    protected:
        virtual ~OSGSceneWorld();

        virtual void onAttachedToScene();
        virtual void onDetachedFromScene();

    private:
        std::ostream& os;
        WorldItemPtr worldItem;
        osg::ref_ptr<OSGCollision> osgCollision;
        boost::signals::connection connectionWithSigCollisionsUpdated;

        void onCollisionsUpdated();
    };

    typedef osg::ref_ptr<OSGSceneWorld> OSGSceneWorldPtr;


    class OSGSceneWorldManager
    {
    public:
        OSGSceneWorldManager();
        ~OSGSceneWorldManager();
            
    private:
        std::ostream& os;
            
        typedef std::map<WorldItemPtr, OSGSceneWorldPtr> SceneWorldMap;
        SceneWorldMap sceneWorlds;

        OSGSceneView* sceneView;
        ItemTreeView* itemTreeView;

        void onItemAdded(Item* item);
        void onWorldItemDetached(WorldItem* worldItem);
        void onWorldItemCheckToggled(WorldItem* worldItem, bool isChecked);
        void showSceneWorld(WorldItem* worldItem, bool show);
    };
}
    
#endif
