/*
  This file is part of Choreonoid, an integrated robotics GUI environment.

  Copyright (C) 2013 Choreonoid Development Team, National Institute of
  Advanced Industrial Science and Technology (AIST), Japan.

  Choreonoid is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  Choreonoid is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with Choreonoid. If not, see <http://www.gnu.org/licenses/>.
*/
  
#ifndef CNOID_BODYPLUGIN_EXPORTDECL_H_INCLUDED
# define CNOID_BODYPLUGIN_EXPORTDECL_H_INCLUDED

// Handle portable symbol export.
// Defining manually which symbol should be exported is required
// under Windows whether MinGW or MSVC is used.
//
// The headers then have to be able to work in two different modes:
// - dllexport when one is building the library,
// - dllimport for clients using the library.
//
// On Linux, set the visibility accordingly. If C++ symbol visibility
// is handled by the compiler, see: http://gcc.gnu.org/wiki/Visibility
# if defined _WIN32 || defined __CYGWIN__
// On Microsoft Windows, use dllimport and dllexport to tag symbols.
#  define CNOID_BODYPLUGIN_DLLIMPORT __declspec(dllimport)
#  define CNOID_BODYPLUGIN_DLLEXPORT __declspec(dllexport)
#  define CNOID_BODYPLUGIN_DLLLOCAL
# else
// On Linux, for GCC >= 4, tag symbols using GCC extension.
#  if __GNUC__ >= 4
#   define CNOID_BODYPLUGIN_DLLIMPORT __attribute__ ((visibility("default")))
#   define CNOID_BODYPLUGIN_DLLEXPORT __attribute__ ((visibility("default")))
#   define CNOID_BODYPLUGIN_DLLLOCAL  __attribute__ ((visibility("hidden")))
#  else
// Otherwise (GCC < 4 or another compiler is used), export everything.
#   define CNOID_BODYPLUGIN_DLLIMPORT
#   define CNOID_BODYPLUGIN_DLLEXPORT
#   define CNOID_BODYPLUGIN_DLLLOCAL
#  endif // __GNUC__ >= 4
# endif // defined _WIN32 || defined __CYGWIN__

# ifdef CNOID_BODYPLUGIN_STATIC
// If one is using the library statically, get rid of
// extra information.
#  define CNOID_BODYPLUGIN_DLLAPI
#  define CNOID_BODYPLUGIN_LOCAL
# else
// Depending on whether one is building or using the
// library define DLLAPI to import or export.
#  ifdef CnoidBodyPlugin_EXPORTS
#   define CNOID_BODYPLUGIN_DLLAPI CNOID_BODYPLUGIN_DLLEXPORT
#  else
#   define CNOID_BODYPLUGIN_DLLAPI CNOID_BODYPLUGIN_DLLIMPORT
#  endif // CNOID_BODYPLUGIN_EXPORTS
#  define CNOID_BODYPLUGIN_LOCAL CNOID_BODYPLUGIN_DLLLOCAL
# endif // CNOID_BODYPLUGIN_STATIC

#endif //! CNOID_BODYPLUGIN_EXPORTDECL_H_INCLUDED

#ifdef CNOID_EXPORT
# undef CNOID_EXPORT
#endif
#define CNOID_EXPORT CNOID_BODYPLUGIN_DLLAPI
