/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_UTIL_TRIANGULATOR_H_INCLUDED
#define CNOID_UTIL_TRIANGULATOR_H_INCLUDED

#include "VRML.h"
#include <boost/dynamic_bitset.hpp>

namespace cnoid {
    
    class Triangulator
    {
    public:

        inline void setVertices(const MFVec3s& vertices) {
            this->vertices = &vertices;
        }

        /**
           @return The number of triangles
        */
        int apply(const std::vector<int>& polygon);

        /**
           Triangulated indices.
           This value is available after calling the 'apply' method.
           The indices are local ones in the polygon index vector given to the apply method.
        */
        inline const std::vector<int>& triangles() {
            return triangles_;
        }

    private:

        enum Convexity { FLAT, CONVEX, CONCAVE };
        
        const MFVec3s* vertices;
        const std::vector<int>* orgPolygon;                                                                  
        std::vector<int> triangles_;
        std::vector<int> workPolygon;
        SFVec3s ccs; // cyclic cross sum
        boost::dynamic_bitset<> earMask;

        inline const SFVec3s& vertex(int localIndex){
            return (*vertices)[(*orgPolygon)[localIndex]];
        }

        inline const SFVec3s& workVertex(int workPolygonIndex){
            return (*vertices)[(*orgPolygon)[workPolygon[workPolygonIndex]]];
        }

        Convexity calcConvexity(int ear);
        bool checkIfEarContainsOtherVertices(int ear);
    };

}

#endif
